LootLog = {
	name = "LootLog",
	slashCommand = "/lootlog",

	-- Default settings
	defaults = {
		verbose = false,
		self = true,
		group = true,
		nameType = 0,
	},

	-- List of items to whitelist as notable
	notableWL = {
		[56862] = true,	-- Fortified Nirncrux
		[56863] = true,	-- Potent Nirncrux
		[68342] = true,	-- Hakeijo
		[139019] = true,	-- Powdered Mother of Pearl
		[139020] = true,	-- Clam Gall
	},

	-- List of items to blacklist from being considered notable
	notableBL = {
		[64713] = true,	-- Laurel
		[69059] = true,	-- Daedric Embers (workaround for a misflag bug)
	},

	nameCache = { },
}

local function OnAddOnLoaded( eventCode, addonName )
	if (addonName ~= LootLog.name) then return end

	EVENT_MANAGER:UnregisterForEvent(LootLog.name, EVENT_ADD_ON_LOADED)

	LootLog.vars = ZO_SavedVars:NewAccountWide("LootLogSavedVariables", 1, nil, LootLog.defaults, nil, "$InstallationWide")

	EVENT_MANAGER:RegisterForEvent(LootLog.name, EVENT_LOOT_RECEIVED, LootLog.OnLootReceived)
	SLASH_COMMANDS[LootLog.slashCommand] = LootLog.HandleSlashCommand
end

local chat = LibChatMessage("LootLog", "LootLog")


function LootLog.OnLootReceived( eventCode, receivedBy, itemName, quantity, itemSound, lootType, self, isPickpocketLoot, questItemIcon, itemId )
	if ( (lootType ~= LOOT_TYPE_ITEM and lootType ~= LOOT_TYPE_COLLECTIBLE) or
	     (self and not LootLog.vars.self) or
	     (not self and not LootLog.vars.group) or
	     (not LootLog.vars.verbose and LootLog.notableBL[itemId]) ) then
		return
	end

	-- Notable items are: any set items, any purple+ items, blue+ special items (e.g., treasure maps)

	local itemType, specializedItemType = GetItemLinkItemType(itemName)
	local itemQuality = GetItemLinkQuality(itemName)
	local itemIsSet = GetItemLinkSetInfo(itemName)

	local itemIsKeyFragment = (itemType == ITEMTYPE_TROPHY) and (specializedItemType == SPECIALIZED_ITEMTYPE_TROPHY_KEY_FRAGMENT)
	local itemIsSpecial = (itemType == ITEMTYPE_TROPHY and not itemIsKeyFragment) or (itemType == ITEMTYPE_COLLECTIBLE) or IsItemLinkConsumable(itemName)

	if ( (LootLog.vars.verbose) or
	     (itemIsSet) or
	     (itemQuality >= ITEM_QUALITY_ARCANE and itemIsSpecial) or
	     (itemQuality >= ITEM_QUALITY_ARTIFACT and not itemIsKeyFragment) or
	     (lootType == LOOT_TYPE_COLLECTIBLE) or
	     (LootLog.notableWL[itemId]) ) then
		LootLog.LogItem(itemName, itemType, quantity, (not self) and receivedBy or nil)
	end
end

function LootLog.LogItem( itemName, itemType, quantity, receivedBy )
	local formattedRecipient
	local formattedQuantity = ""
	local formattedTrait = ""

	if (not receivedBy) then
		formattedRecipient = GetString(SI_LOOTLOG_SELF_IDENTIFIER)
	else
		-- Create a character link to make it easier to contact the recipient

		-- Name formatting options:
		-- 0: Conform to user's settings in the base game (default)
		-- 1: Show the character name
		-- 2: Show the account name
		-- 3: Show both

		local nameType = (LootLog.vars.nameType ~= 0) and LootLog.vars.nameType or (ZO_ShouldPreferUserId() and 2 or 1)

		local accountName = (nameType ~= 1) and LootLog.GetAccountName(receivedBy) or nil
		local charName = LocalizeString("<<1>>", receivedBy)

		if (not accountName) then
			formattedRecipient = string.format("|H0:character:%s|h%s|h", receivedBy, charName)
		elseif (nameType == 2) then
			formattedRecipient = string.format("|H0:character:%s|h%s|h", accountName, accountName)
		else
			formattedRecipient = string.format("|H1:character:%s|h[%s%s]|h", receivedBy, charName, accountName)
		end
	end

	if (quantity > 1) then
		formattedQuantity = string.format("|cFFFFFFx%d|r", quantity)
	end

	local traitType = GetItemLinkTraitInfo(itemName)

	if (traitType ~= ITEM_TRAIT_TYPE_NONE and itemType ~= ITEMTYPE_ARMOR_TRAIT and itemType ~= ITEMTYPE_WEAPON_TRAIT) then
		formattedTrait = string.format(" |cFFFFFF(%s)|r", GetString("SI_ITEMTRAITTYPE", traitType))
	end

	LootLog.Msg(string.format(
		"%s%s%s → |c%06X%s|r",
		itemName:gsub("^|H0", "|H1", 1),
		formattedQuantity,
		formattedTrait,
		HashString(formattedRecipient) % 0x1000000,	-- Use the hash of the name for the color so that is random, but consistent
		formattedRecipient
	))
end

function LootLog.GetAccountName( character )
	local accountName = LootLog.nameCache[character]

	if (not accountName) then
		for i = 1, GetGroupSize() do
			local unitTag = GetGroupUnitTagByIndex(i)
			if (GetRawUnitName(unitTag) == character) then
				accountName = GetUnitDisplayName(unitTag)
				LootLog.nameCache[character] = accountName
				break
			end
		end
	end

	return(accountName)
end

function LootLog.Msg( text )
	chat:Print(text)
end

function LootLog.HandleSlashCommand( command )
	if (command == "verbose") then
		LootLog.vars.verbose = not LootLog.vars.verbose
		LootLog.Msg(GetString("SI_LOOTLOG_VERBOSE_STATUS", LootLog.vars.verbose and 1 or 0))
	elseif (command == "self") then
		LootLog.vars.self = not LootLog.vars.self
		LootLog.Msg(GetString("SI_LOOTLOG_SELF_STATUS", LootLog.vars.self and 1 or 0))
	elseif (command == "group") then
		LootLog.vars.group = not LootLog.vars.group
		LootLog.Msg(GetString("SI_LOOTLOG_GROUP_STATUS", LootLog.vars.group and 1 or 0))
	elseif (command == "name") then
		LootLog.vars.nameType = (LootLog.vars.nameType + 1) % 4
		LootLog.Msg(GetString("SI_LOOTLOG_NAMETYPE_STATUS", LootLog.vars.nameType))
	else
		LootLog.Msg(GetString(SI_LOOTLOG_LIST_OF_COMMANDS))
	end
end

EVENT_MANAGER:RegisterForEvent(LootLog.name, EVENT_ADD_ON_LOADED, OnAddOnLoaded)
